#pragma once

#include <unordered_map>

#include "BinarySocket.h"
#include "BinaryThread.h"

enum CommandRet
{
    COMMAND_RET_OK = 0,
    COMMAND_RET_NG,
    COMMAND_RET_GENERIC_OK,
    COMMAND_RET_GENERIC_NG
};

enum BinaryFuncBaseRet
{
    BINARY_FUNC_BASE_RET_OK = 0,
    BINARY_FUNC_BASE_RET_NG
};

class BinaryThreadPool
{
    public:
        BinaryThreadPool();
        ~BinaryThreadPool();

        void AssignTask(FuncItem i_func_item);
        void RegisterCallbackFunc(std::function<void(CommandRet, std::string, int, std::vector<std::string>)> i_function);

    private:
        std::unique_ptr<std::thread> m_pthread;
        bool m_stop_task;
        std::mutex m_mutex;
        std::condition_variable m_cv;
        std::vector<std::unique_ptr<BinaryThread>> m_thread_pool;
        std::vector<FuncItem> m_func_list;
        std::function<void(CommandRet, std::string, int, std::vector<std::string>)> m_function;

    private:
        void Pri_GetFuncReturn(FuncRet i_ret, FuncItem i_func_item, std::vector<std::string> i_msg);
        bool Pri_IsTaskPending();
        void Pri_StopTask();
        void Pri_TaskThread();
};

class BinaryFuncBase
{
    public:
        BinaryFuncBase();
        ~BinaryFuncBase();

    public:
        void Initialize(std::string i_module_name);

    protected:
        using CommandFunc = FuncRet(BinaryFuncBase::*)(std::vector<std::string>, std::vector<std::string>&);
        struct FuncTableItem
        {
            CommandFunc func;
            int arg_num;
        };

    protected:
        void AddNewCommand(std::string i_command, CommandFunc i_func, int i_arg_num);

    private:
        std::string m_module_name;
        std::unordered_map<std::string, FuncTableItem> m_func_table;
        std::unordered_map<std::string, FuncTableItem> m_generic_func_table;
        std::unique_ptr<BinarySocket> m_pBinarySocket;
        std::unique_ptr<BinaryThreadPool> m_pBinaryThreadPool;

    private:
        BinaryFuncBaseRet Pri_ParseCommand(std::string i_command, FuncItem &o_func_item, std::vector<std::string> &o_ret_msg);
        BinaryFuncBaseRet Pri_StringSplit(std::string i_string, char i_delimiter, std::vector<std::string> &o_string_array);
        void Pri_AddNewGenericCommand(std::string i_command, CommandFunc i_func, int i_arg_num);
        void Pri_ReceiveCommand(std::string i_command);
        void Pri_PushTaskToThread(FuncItem i_func_item);
        void Pri_SendReceiveMessage(std::string i_command);
        void Pri_SendResultMessage(CommandRet i_enum, std::string i_command, int i_id, std::vector<std::string> i_value);
        void Pri_SendReplyMessage(std::string i_string);

    private:
        virtual FuncRet Pri_ReinitModule(std::vector<std::string> i_arg, std::vector<std::string> &o_msg) { return BINARY_FUNC_RET_OK; }
        FuncRet Pri_GetModuleName(std::vector<std::string> i_arg, std::vector<std::string> &o_msg);
};