#include <iostream>
#include <unistd.h>

#include "BinaryThread.h"

BinaryThread::BinaryThread(int i_thread_id) :
    mp_thread(nullptr),
    m_function(nullptr),
    m_is_busy(false),
    m_force_stop(false),
    m_thread_id(i_thread_id)
{
    Pri_Initialize();
}

BinaryThread::~BinaryThread()
{
    Pri_ForceStop();
    if (mp_thread)
        mp_thread->join();
}

void BinaryThread::RegisterCallbackFunc(std::function<void(FuncRet, FuncItem, std::vector<std::string>)> i_function)
{
    m_function = i_function;
}

int BinaryThread::GetThreadId()
{
    return m_thread_id;
}

bool BinaryThread::IsThreadBusy()
{
    return m_is_busy;
}

void BinaryThread::Pri_ForceStop()
{
    m_force_stop = true;
    m_cv.notify_one();
}

bool BinaryThread::AssignTask(FuncItem i_task)
{
    // std::cout << "%%%%%%%%%%" << __FILE__ << " " << __FUNCTION__ << std::endl;
    if (IsThreadBusy())
    {
        return false;
    }

    std::unique_lock<std::mutex> lck(m_mutex);
    m_is_busy = true;
    m_task = i_task;
    m_cv.notify_one();
    return true;
}

void BinaryThread::Pri_Initialize()
{
    // std::cout << "%%%%%%%%%%" << __FILE__ << " " << __FUNCTION__ << std::endl;
    if (mp_thread)
    {
        return;
    }
    mp_thread = std::unique_ptr<std::thread>(new std::thread(&BinaryThread::Pri_ThreadTask, this));
}

void BinaryThread::Pri_ThreadTask()
{
    while(true)
    {
        // std::cout << "%%%%%%%%%%" << __FILE__ << " " << __FUNCTION__ << std::endl;
        std::unique_lock<std::mutex> lck(m_mutex);
        m_cv.wait(lck);

        if (m_force_stop)
            break;

        FuncRet ret = BINARY_FUNC_RET_NG;
        std::vector<std::string> ret_msg;
        std::cout << "Task handled by Thread " << std::to_string(m_thread_id) << std::endl;
        if (m_task.func)
            ret = m_task.func(m_task.input_arg, ret_msg);
        else
            ret_msg.push_back("Invalid function pointer");

        if (m_function)
            m_function(ret, m_task, ret_msg);

        m_is_busy = false;
        lck.unlock();
    }
}