#pragma once

#include <mutex>
#include <vector>
#include <condition_variable>
#include <thread>

enum FuncRet
{
    BINARY_FUNC_RET_OK = 0,
    BINARY_FUNC_RET_NG
};

enum FuncType
{
    BINARY_FUNC_GENERIC,
    BINARY_FUNC_UNIQUE,
    BINARY_FUNC_INVALID
};

struct FuncItem
{
    FuncType func_type;
    std::string command;
    std::function<FuncRet(std::vector<std::string>, std::vector<std::string>&)> func;
    unsigned long id;
    std::vector<std::string> input_arg; //Add init dfault value

    FuncItem ()
        : func_type(BINARY_FUNC_INVALID),
          command(""),
          func(nullptr),
          id(0)
    {
        input_arg.clear();
    }
};

class BinaryThread
{
    public:
        BinaryThread(int i_thread_id);
        ~BinaryThread();

        void RegisterCallbackFunc(std::function<void(FuncRet, FuncItem, std::vector<std::string>)> i_function);
        int GetThreadId();
        bool IsThreadBusy();
        bool AssignTask(FuncItem i_task);

    private:
        std::unique_ptr<std::thread> mp_thread;
        std::function<void(FuncRet, FuncItem, std::vector<std::string>)> m_function;
        FuncItem m_task;
        bool m_is_busy;
        bool m_force_stop;
        int m_thread_id;
        std::mutex m_mutex;
        std::condition_variable m_cv;

    private:
        void Pri_ForceStop();
        void Pri_Initialize();
        void Pri_ThreadTask();
};